#! /usr/bin/env python3.8
# -*- coding: utf8 -*-

# pwselect – Python Web Scraper for the Selection of Electricity Companies
# analysiert Daten von Stromvergleich.de
# Copyright (C) 2022  B. Sc. Felix Krickl

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

# Programm	:	pwselect
# Version   :   1.1
# lang      :   de
# Autor     :	Felix Krickl

import argparse
import csv
import datetime
import json
import requests
import sys
from operator import itemgetter

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    strings                           ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
ende = "pwselect wurde ohne Fehler beendet."
help_verbrauch = "Ihr Jahresverbrauch in kWh als Integer."
help_plz = "Ihre Postleitzahl als Integer."
help_garantie = "Geben Sie in Monaten an, wie lange der Preis garantiert sein soll. Default: 12."
help_orts_id = "In manchen Fällen ist die Postleitzahl nicht eindeutig und " \
               "es muss daher eine zusätzliche ID vom Typ Integer angegeben werden."
help_dateiname = "Geben Sie einen beliebigen Dateinamen an, in den der csv-formatierte output geschrieben wird. " \
                 "Default: Das aktuelle Datum + Uhrzeit."
help_verbose = "In der ausgegebenen Datei wird ein 3-zeiliger Header hinzugefügt, " \
               "der Auskunft über das verwendete Programm, die verwendete URL und den Zeitpunkt der Abfrage gibt. " \
               "Default: False."
nichts_geschrieben = "pwselect wurde beendet.\nEs wurde keine Datei geschrieben."
ort_id_gebraucht = "Ihre Postleitzahl liefert auf Stromvergleich.de kein eindeutiges Ergebnis.\n" \
                   "Sie müssen eine der folgenden IDs mit der Option '-o' angeben:\n"
pwselect_descr = "pwselect ist ein Web Scraper für Stromvergleich.de. " \
                    "Die Tarife werden in Abhängigkeit der eigenen Postleitzahl und des eigenen Jahresverbrauchs " \
                    "analysiert und nach dem individuell bestimmten Monatspreis aufsteigend sortiert. " \
                    "Die Ausgabe wird im csv-Format geschrieben.\n"
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    sets                              ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
choices_garantee=[3, 6, 12]

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    kommandozeilen-schnittstelle      ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
parser = argparse.ArgumentParser(description=pwselect_descr, prog="pwselect")
parser.add_argument('-v', '--verbrauch', default=1500, type=int, help=help_verbrauch, dest='verbrauch')
parser.add_argument('-p', '--plz', required=True, type=int, help=help_plz, dest='plz')
parser.add_argument('--preis-garantie', default=12, type=int, help=help_garantie, choices=choices_garantee,
                        dest='garantie')
parser.add_argument('-o', '--orts-id', type=int, help=help_orts_id, dest='ort_id')
parser.add_argument('-d', '--dateiname', default=str(datetime.datetime.now().strftime("%Y-%m-%d_%I-%M")) + ".csv",
                    type=str, help=help_dateiname, dest='dateiname')
parser.add_argument('--verbose', default=False, action='store_true', help=help_verbose, dest='verbose')

class C():
    pass

c = C()
args = parser.parse_args(namespace=c)
verbrauch = c.verbrauch
plz = c.plz
garantie = c.garantie
ort_id = c.ort_id
dateiname = c.dateiname
verbose = c.verbose
zeitstempel = str(datetime.datetime.now().strftime("%Y-%m-%d_%I-%M"))

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    arbeits-url generieren            ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
stamm_url = "https://www.stromvergleich.de/stromrechner?priceGuaranteeMinDuration="
if ort_id==None:
    arbeits_url = stamm_url + f"{garantie}&consumption={verbrauch}&zip={plz}&output=offers"
else:
    arbeits_url = stamm_url + f"{garantie}&locationId={ort_id}&consumption={verbrauch}&zip={plz}&output=offers"
print("Verwende URL: " + arbeits_url)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    parsen                            ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
result = requests.get(arbeits_url).text
parsed = json.loads(result)
try:
    angebote = parsed['offers']
except KeyError:
    print(ort_id_gebraucht)
    ort_id_auswahl = [parsed['locations'][ort]['id'] for ort in range(len(parsed['locations']))]
    ort_name_auswahl = [parsed['locations'][ort]['name'] for ort in range(len(parsed['locations']))]
    for ort in range(len(ort_id_auswahl)):
        print(ort_name_auswahl[ort] + "\t:\t" + ort_id_auswahl[ort])
    print()
    sys.exit("Sie können auch die Hilfe mit der Option '-h' aufrufen.\n" + ende + nichts_geschrieben)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    auslesen & schreiben in csv       ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
tabelle = []
for angebot in range(len(angebote)):
    stamm = angebote[angebot]
#           infos holen:
    provider = f"{stamm['provider']['name']}".replace(' ', '_')
    tarif = f"{stamm['tariff']['name']}".replace(' ', '_')
    grund_preis_jahr = f"{stamm['price']['basicPrice']['amount']}"
    grund_preis_monat = float(grund_preis_jahr)/12
    arbeits_preis_cent = f"{stamm['price']['workingPrice'][0]['amount']}"
    arbeits_preis_euro = float(arbeits_preis_cent)/100
    monatspreis = grund_preis_monat + (arbeits_preis_euro * float(verbrauch)/12)

    try:
        if f"{stamm['contract']['duration']['unit']}".__eq__("month"):
            laufzeit = f"{stamm['contract']['duration']['amount']}"
        elif f"{stamm['contract']['duration']['unit']}".__eq__("year"):
            laufzeit = int(f"{stamm['contract']['duration']['amount']}") * 12
        else:
            laufzeit = f"{stamm['contract']['duration']['amount']}-{stamm['contract']['duration']['unit']}"
    except TypeError:
        laufzeit = "k.A."

    try:
        if f"{stamm['contract']['prolongation']['unit']}".__eq__("month"):
            verlaengerung = f"{stamm['contract']['prolongation']['amount']}"
        elif f"{stamm['contract']['prolongation']['unit']}".__eq__("year"):
            verlaengerung = int(f"{stamm['contract']['prolongation']['amount']}") * 12
        else:
            verlaengerung = f"{stamm['contract']['prolongation']['amount']}-{stamm['contract']['prolongation']['unit']}"
    except TypeError:
        verlaengerung = "k.A."

    try:
        if f"{stamm['contract']['cancellation']['unit']}".__eq__("month"):
            kuendigungsfrist = f"{stamm['contract']['cancellation']['amount']}"
        elif f"{stamm['contract']['cancellation']['unit']}".__eq__("week"):
            kuendigungsfrist = float(f"{stamm['contract']['cancellation']['amount']}")/4
        else:
            kuendigungsfrist = f"{stamm['contract']['cancellation']['amount']}-{stamm['contract']['cancellation']['unit']}"
    except TypeError:
        kuendigungsfrist = "k.A."

#           infos an zeile-dictionary anhängen
    zeile = [provider, tarif, round(grund_preis_monat, 2), round(arbeits_preis_euro, 4), round(monatspreis, 2),
             laufzeit, verlaengerung, kuendigungsfrist]
    tabelle.append(zeile)

tabelle_sortiert = sorted(tabelle, key=itemgetter(4))   # nach Monatspreis aufsteigend sortiert

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    Datei schreiben                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
with open(dateiname, 'w', newline='\n') as datei:
    writer = csv.writer(datei, delimiter='\t')
    if verbose:
        writer.writerow(['# Stromvergleich.de Tarife, bereitgestellt durch pwselect'])
        writer.writerow(['# Verwendete URL:', f'{arbeits_url}'])
        writer.writerow(['# Zeitstempel der Abfrage:', f'{zeitstempel}'])
    writer.writerow(['#Anbieter', 'Tarif', 'Grundpreis', 'Arbeitspreis', 'Monatspreis', 'Laufzeit', 'Verlängerung',
                        'Kündigungsfrist'])
    writer.writerow(['#', '', '€/Monat', '€/kWh', f'bei {verbrauch} kWh/Jahr', 'Monate', 'Monate', 'Monate'])
    for sortierte_zeile in range(len(tabelle_sortiert)):
        writer.writerow(tabelle_sortiert[sortierte_zeile])

sys.exit(ende)
